@@ -1,5 +1,5 @@
 /* tc-ia64.h -- Header file for tc-ia64.c.
-   Copyright (C) 1998-2020 Free Software Foundation, Inc.
+   Copyright (C) 1998-2025 Free Software Foundation, Inc.
    Contributed by David Mosberger-Tang <davidm@hpl.hp.com>
 
    This file is part of GAS, the GNU Assembler.
@@ -92,7 +92,7 @@ struct ia64_fix
     enum ia64_opnd opnd;
   };
 
-extern void ia64_end_of_source (void);
+extern void ia64_md_finish (void);
 extern void ia64_start_line (void);
 extern int ia64_unrecognized_line (int);
 extern void ia64_frob_label (struct symbol *);
@@ -122,7 +122,7 @@ extern void ia64_check_label (symbolS *);
 extern int ia64_estimate_size_before_relax (fragS *, asection *);
 extern void ia64_convert_frag (fragS *);
 
-#define md_end()       			ia64_end_of_source ()
+#define md_finish()       			ia64_md_finish ()
 #define md_start_line_hook()		ia64_start_line ()
 #define tc_unrecognized_line(ch)	ia64_unrecognized_line (ch)
 #define tc_frob_label(s)		ia64_frob_label (s)
