@@ -1,4 +1,4 @@
-/* Copyright (C) 2007-2023 Free Software Foundation, Inc.
+/* Copyright (C) 2007-2025 Free Software Foundation, Inc.
 
    This file is part of the GNU opcodes library.
 
@@ -30,7 +30,9 @@
 
 /* Build-time checks are preferrable over runtime ones.  Use this construct
    in preference where possible.  */
+#ifndef static_assert
 #define static_assert(e) ((void)sizeof (struct { int _:1 - 2 * !(e); }))
+#endif
 
 static const char *program_name = NULL;
 static int debug = 0;
@@ -45,7 +47,7 @@ typedef struct dependency
 static const dependency isa_dependencies[] =
 {
   { "UNKNOWN",
-    "~IAMCU" },
+    "~(IAMCU|MPX)" },
   { "GENERIC32",
     "386" },
   { "GENERIC64",
@@ -94,6 +96,8 @@ static const dependency isa_dependencies[] =
     "ZNVER2|INVLPGB|TLBSYNC|VAES|VPCLMULQDQ|INVPCID|SNP|OSPKE" },
   { "ZNVER4",
     "ZNVER3|AVX512F|AVX512DQ|AVX512IFMA|AVX512CD|AVX512BW|AVX512VL|AVX512_BF16|AVX512VBMI|AVX512_VBMI2|AVX512_VNNI|AVX512_BITALG|AVX512_VPOPCNTDQ|GFNI|RMPQUERY" },
+  { "ZNVER5",
+    "ZNVER4|AVX_VNNI|MOVDIRI|MOVDIR64B|AVX512_VP2INTERSECT|PREFETCHI" },
   { "BTVER1",
     "GENERIC64|FISTTP|MONITOR|CX16|LAHF_SAHF|Rdtscp|SSSE3|SSE4A|ABM|PRFCHW|Clflush|FISTTP|SVME" },
   { "BTVER2",
@@ -239,6 +243,8 @@ static const dependency isa_dependencies[] =
   { "AVX10_1",
     "AVX512VL|AVX512DQ|AVX512CD|AVX512VBMI|AVX512_VBMI2|AVX512IFMA"
     "|AVX512_VNNI|AVX512_BF16|AVX512_FP16|AVX512_VPOPCNTDQ|AVX512_BITALG" },
+  { "AVX10_2",
+    "AVX10_1" },
   { "SEV_ES",
     "SVME" },
   { "SNP",
@@ -259,6 +265,12 @@ static const dependency isa_dependencies[] =
     "AMX_TILE" },
   { "AMX_COMPLEX",
     "AMX_TILE" },
+  { "AMX_TRANSPOSE",
+    "AMX_TILE" },
+  { "AMX_TF32",
+    "AMX_TILE" },
+  { "AMX_FP8",
+    "AMX_TILE" },
   { "KL",
     "SSE2" },
   { "WIDEKL",
@@ -275,6 +287,10 @@ static const dependency isa_dependencies[] =
     "64" },
   { "USER_MSR",
     "64" },
+  { "MSR_IMM",
+    "64" },
+  { "APX_F",
+    "XSAVE|64" },
 };
 
 /* This array is populated as process_i386_initializers() walks cpu_flags[].  */
@@ -328,6 +344,7 @@ static bitfield cpu_flags[] =
   BITFIELD (3dnow),
   BITFIELD (3dnowA),
   BITFIELD (PadLock),
+  BITFIELD (GMI),
   BITFIELD (SVME),
   BITFIELD (VMX),
   BITFIELD (SMX),
@@ -397,6 +414,9 @@ static bitfield cpu_flags[] =
   BITFIELD (FRED),
   BITFIELD (LKGS),
   BITFIELD (USER_MSR),
+  BITFIELD (MSR_IMM),
+  BITFIELD (APX_F),
+  BITFIELD (AVX10_2),
   BITFIELD (MWAITX),
   BITFIELD (CLZERO),
   BITFIELD (OSPKE),
@@ -417,6 +437,9 @@ static bitfield cpu_flags[] =
   BITFIELD (AMX_BF16),
   BITFIELD (AMX_FP16),
   BITFIELD (AMX_COMPLEX),
+  BITFIELD (AMX_TRANSPOSE),
+  BITFIELD (AMX_TF32),
+  BITFIELD (AMX_FP8),
   BITFIELD (AMX_TILE),
   BITFIELD (MOVDIRI),
   BITFIELD (MOVDIR64B),
@@ -482,7 +505,11 @@ static bitfield opcode_modifiers[] =
   BITFIELD (Disp8MemShift),
   BITFIELD (Optimize),
   BITFIELD (Dialect),
+  BITFIELD (IntelSuffix),
   BITFIELD (ISA64),
+  BITFIELD (NoEgpr),
+  BITFIELD (NF),
+  BITFIELD (Rex2),
 };
 
 #define CLASS(n) #n, n
@@ -601,7 +628,7 @@ static void
 process_copyright (FILE *fp)
 {
   fprintf (fp, "/* This file is automatically generated by i386-gen.  Do not edit!  */\n\
-/* Copyright (C) 2007-2023 Free Software Foundation, Inc.\n\
+/* Copyright (C) 2007-2025 Free Software Foundation, Inc.\n\
 \n\
    This file is part of the GNU opcodes library.\n\
 \n\
@@ -851,7 +878,16 @@ output_cpu_flags (FILE *table, bitfield *flags, unsigned int size,
 	active_cpu_flags.array[i / 32] |= 1U << (i % 32);
     }
 
-  fprintf (table, "%d } }%s\n", flags[i].value, comma);
+#if defined(CpuAttrUnused) != defined(CpuUnused)
+  if (mode <= 0)
+# ifdef CpuUnused
+    fprintf (table, " } }%s\n", comma);
+# else
+    fprintf (table, "%d, 0 } }%s\n", flags[i].value, comma);
+# endif
+  else
+#endif
+    fprintf (table, "%d } }%s\n", flags[i].value, comma);
 }
 
 static void
@@ -1069,12 +1105,47 @@ get_element_size (char **opnd, int lineno)
   return elem_size;
 }
 
+static bool
+rex2_disallowed (const unsigned long long opcode, unsigned int length,
+		 unsigned int space, const char *cpu_flags)
+{
+  /* Some opcodes encode a ModR/M-like byte directly in the opcode.  */
+  unsigned int base_opcode = opcode >> (8 * length - 8);
+
+  /* All opcodes listed map0 0x4*, 0x7*, 0xa*, 0xe* and map1 0x3*, 0x8*
+     are reserved under REX2 and triggers #UD when prefixed with REX2 */
+  if (space == 0)
+    switch (base_opcode >> 4)
+      {
+      case 0x4:
+      case 0x7:
+      case 0xA:
+      case 0xE:
+	return true;
+      default:
+	return false;
+    }
+
+  if (space == SPACE_0F)
+    switch (base_opcode >> 4)
+      {
+      case 0x3:
+      case 0x8:
+	return true;
+      default:
+	return false;
+      }
+
+  return false;
+}
+
 static void
 process_i386_opcode_modifier (FILE *table, char *mod, unsigned int space,
 			      unsigned int prefix, const char *extension_opcode,
-			      char **opnd, int lineno)
+			      char **opnd, int lineno, bool rex2_disallowed)
 {
   char *str, *next, *last;
+  bool disp8_shift_derived = false;
   bitfield modifiers [ARRAY_SIZE (opcode_modifiers)];
   static const char *const spaces[] = {
 #define SPACE(n) [SPACE_##n] = #n
@@ -1082,9 +1153,10 @@ process_i386_opcode_modifier (FILE *table, char *mod, unsigned int space,
     SPACE(0F),
     SPACE(0F38),
     SPACE(0F3A),
-    SPACE(EVEXMAP5),
-    SPACE(EVEXMAP6),
-    SPACE(VEXMAP7),
+    SPACE(MAP4),
+    SPACE(MAP5),
+    SPACE(MAP6),
+    SPACE(MAP7),
     SPACE(XOP08),
     SPACE(XOP09),
     SPACE(XOP0A),
@@ -1138,7 +1210,10 @@ process_i386_opcode_modifier (FILE *table, char *mod, unsigned int space,
 	      if (strcasecmp(str, "Broadcast") == 0)
 		val = get_element_size (opnd, lineno) + BYTE_BROADCAST;
 	      else if (strcasecmp(str, "Disp8MemShift") == 0)
-		val = get_element_size (opnd, lineno);
+		{
+		  val = get_element_size (opnd, lineno);
+		  disp8_shift_derived = true;
+		}
 
 	      set_bitfield (str, modifiers, val, ARRAY_SIZE (modifiers),
 			    lineno);
@@ -1191,13 +1266,27 @@ process_i386_opcode_modifier (FILE *table, char *mod, unsigned int space,
 
   /* Rather than evaluating multiple conditions at runtime to determine
      whether an EVEX encoding is being dealt with, derive that information
-     right here.  A missing EVex attribute means "dynamic".  */
-  if (!modifiers[EVex].value
-      && (modifiers[Disp8MemShift].value
-	  || modifiers[Broadcast].value
+     right here.  A missing EVex attribute means "dynamic".  There's one
+     exception though: A value-less Disp8MemShift needs zapping rather than
+     respecting if no other attribute indicates EVEX encoding.  This is for
+     certain SSE2AVX templatized templates to work reasonably.  */
+  if (!modifiers[EVex].value)
+    {
+      if (modifiers[Broadcast].value
 	  || modifiers[Masking].value
-	  || modifiers[SAE].value))
-    modifiers[EVex].value = EVEXDYN;
+	  || modifiers[SAE].value)
+	modifiers[EVex].value = EVEXDYN;
+      else if (disp8_shift_derived)
+	modifiers[Disp8MemShift].value = 0;
+      else if (modifiers[Disp8MemShift].value)
+	modifiers[EVex].value = EVEXDYN;
+    }
+
+  /* Vex, legacy map2 and map3 and rex2_disallowed do not support EGPR.
+     For templates supporting both Vex and EVex allowing EGPR.  */
+  if ((modifiers[Vex].value || space > SPACE_0F || rex2_disallowed)
+      && !modifiers[EVex].value)
+    modifiers[NoEgpr].value = 1;
 
   output_opcode_modifier (table, modifiers, ARRAY_SIZE (modifiers));
 }
@@ -1423,7 +1512,9 @@ output_i386_opcode (FILE *table, const char *name, char *str,
   free (ident);
 
   process_i386_opcode_modifier (table, opcode_modifier, space, prefix,
-				extension_opcode, operand_types, lineno);
+				extension_opcode, operand_types, lineno,
+				rex2_disallowed (opcode, length, space,
+						 cpu_flags));
 
   process_i386_cpu_flag (table, cpu_flags, NULL, ",", "    ", lineno, CpuMax);
 
@@ -1478,10 +1569,10 @@ opcode_hash_eq (const void *p, const void *q)
   return strcmp (name, entry->name) == 0;
 }
 
-static void
+static bool
 parse_template (char *buf, int lineno)
 {
-  char sep, *end, *name;
+  char sep, *end, *ptr;
   struct template *tmpl;
   struct template_instance *last_inst = NULL;
 
@@ -1508,8 +1599,16 @@ parse_template (char *buf, int lineno)
 	prev->next = tmpl->next;
       else
 	templates = tmpl->next;
-      return;
+      return true;
     }
+
+  /* Check whether this actually is a reference to an existing template:
+     If there's '>' ahead of ':', it can't be a new template definition
+     (and template undefs have are dealt with above).  */
+  ptr = strchr (buf, '>');
+  if (ptr != NULL && ptr < end)
+    return false;
+
   *end++ = '\0';
   remove_trailing_whitespaces (buf);
 
@@ -1584,6 +1683,71 @@ parse_template (char *buf, int lineno)
 
   tmpl->next = templates;
   templates = tmpl;
+
+  return true;
+}
+
+static void
+expand_template (const struct template *tmpl,
+		 const struct template_instance *inst,
+		 char *dst, const char *src, int lineno)
+{
+  while (*src)
+    {
+      const char *ident = tmpl->name, *end;
+      const struct template_param *param;
+      const struct template_arg *arg;
+
+      if ((*dst = *src++) != '<')
+	{
+	  ++dst;
+	  continue;
+	}
+      while (ISSPACE(*src))
+	++src;
+      while (*ident && *src == *ident)
+	++src, ++ident;
+      while (ISSPACE(*src))
+	++src;
+      if (*src != ':' || *ident != '\0')
+	{
+	  memcpy (++dst, tmpl->name, ident - tmpl->name);
+	  dst += ident - tmpl->name;
+	  continue;
+	}
+      while (ISSPACE(*++src))
+	;
+
+      end = src;
+      while (*end != '\0' && !ISSPACE(*end) && *end != '>')
+	++end;
+
+      for (param = tmpl->params, arg = inst->args; param;
+	   param = param->next, arg = arg->next)
+	{
+	  if (end - src == strlen (param->name)
+	      && !memcmp (src, param->name, end - src))
+	    {
+	      src = end;
+	      break;
+	    }
+	}
+
+      if (param == NULL)
+	fail ("template '%s' has no parameter '%.*s'\n",
+	      tmpl->name, (int)(end - src), src);
+
+      while (ISSPACE(*src))
+	++src;
+      if (*src != '>')
+	fail ("%s: %d: missing '>'\n", filename, lineno);
+
+      memcpy(dst, arg->val, strlen(arg->val));
+      dst += strlen(arg->val);
+      ++src;
+    }
+
+  *dst = '\0';
 }
 
 static unsigned int
@@ -1660,71 +1824,20 @@ expand_templates (char *name, const char *str, htab_t opcode_hash_table,
 
       for (inst = tmpl->instances; inst; inst = inst->next)
 	{
-	  char *name2 = xmalloc(strlen(name) + strlen(inst->name) + strlen(ptr2) + 1);
+	  char *name2 = xmalloc(strlen(name) + strlen(inst->name)
+			+ 2 * strlen(ptr2) + 1);
 	  char *str2 = xmalloc(2 * strlen(str));
-	  const char *src;
 
-	  strcpy (name2, name);
-	  strcat (name2, inst->name);
-	  strcat (name2, ptr2);
+	  ptr1 = stpcpy (name2, name);
+	  ptr1 = stpcpy (ptr1, inst->name);
 
-	  for (ptr1 = str2, src = str; *src; )
-	    {
-	      const char *ident = tmpl->name, *end;
-	      const struct template_param *param;
-	      const struct template_arg *arg;
+	  /* Expand this template in trailing portion of mnemonic.  */
+	  expand_template (tmpl, inst, ptr1, ptr2, lineno);
 
-	      if ((*ptr1 = *src++) != '<')
-		{
-		  ++ptr1;
-		  continue;
-		}
-	      while (ISSPACE(*src))
-		++src;
-	      while (*ident && *src == *ident)
-		++src, ++ident;
-	      while (ISSPACE(*src))
-		++src;
-	      if (*src != ':' || *ident != '\0')
-		{
-		  memcpy (++ptr1, tmpl->name, ident - tmpl->name);
-		  ptr1 += ident - tmpl->name;
-		  continue;
-		}
-	      while (ISSPACE(*++src))
-		;
-
-	      end = src;
-	      while (*end != '\0' && !ISSPACE(*end) && *end != '>')
-		++end;
-
-	      for (param = tmpl->params, arg = inst->args; param;
-		   param = param->next, arg = arg->next)
-		{
-		  if (end - src == strlen (param->name)
-		      && !memcmp (src, param->name, end - src))
-		    {
-		      src = end;
-		      break;
-		    }
-		}
-
-	      if (param == NULL)
-		fail ("template '%s' has no parameter '%.*s'\n",
-		      tmpl->name, (int)(end - src), src);
-
-	      while (ISSPACE(*src))
-		++src;
-	      if (*src != '>')
-		fail ("%s: %d: missing '>'\n", filename, lineno);
-
-	      memcpy(ptr1, arg->val, strlen(arg->val));
-	      ptr1 += strlen(arg->val);
-	      ++src;
-	    }
-
-	  *ptr1 = '\0';
+	  /* Expand this template in attributes and operands.  */
+	  expand_template (tmpl, inst, str2, str, lineno);
 
+	  /* Expand further templates, if any. */
 	  expand_templates (name2, str2, opcode_hash_table, opcode_array_p,
 			    lineno);
 
@@ -1840,10 +1953,12 @@ process_i386_opcodes (FILE *table)
 	  /* Ignore comments.  */
 	case '\0':
 	  continue;
-	  break;
+
 	case '<':
-	  parse_template (p, lineno);
-	  continue;
+	  if (parse_template (p, lineno))
+	    continue;
+	  break;
+
 	default:
 	  if (!marker)
 	    continue;
